#include    "MTypes.h"
#include    "ComplexMac.h"
/*######################################################################*/
//  Ver 28-08-2007
/*----------------------------------------------------------------------*/
//  This routines executes fast Fourier transform
//  on a given 1-dimensional array of complex numbers.
//  Complex array is located in array data[2*dim]:
//  Real(h[k]) == data[2*k  ]
//  Imag(h[k]) == data[2*k+1]
//  where 0 <= k < dim.
//  dim MUST be integer power of 2, and MUST be equal or greater than 16.
//  The parameter sign should be 0 for normal transform
//  or 1 for inverse Fourier transform.
//  Basic formula:
//  H[n] = SUM{ exp(sign*2*Pi*i*n*k/dim) * h[k], 0<=k<dim }
//  Calculated values of H[n] replace h[n] in data[].
//  Being applied sequentially to the same array,
//  transforms with sign=0, then with sign=1, the routine
//  restores initial data, multiplied by dim. 
/*======================================================================*/
#pragma intrinsic   (sin, cos, sqrt)

#define dar(i)      data[(i)*2+0]                   // Real part
#define dai(i)      data[(i)*2+1]                   // Imag part

#define tcos(i)     sc[(i)*2+0]                     // Cos value
#define tsin(i)     sc[(i)*2+1]                     // Sin value

#define scc(i)      sc[(i)]                         // Cos value
#define scs(i)      sc[(i)+1]                       // Sin value

//  w = (wr(cos), wi(sin))
#define BflyN(a, b, w)                              \
{                                                   \
    real tr = real(w##r*dar(b) - w##i*dai(b));      \
    real ti = real(w##r*dai(b) + w##i*dar(b));      \
    dar(b)  = dar(a) - tr; dar(a) += tr;            \
    dai(b)  = dai(a) - ti; dai(a) += ti;            \
}
//  w = (1, 0)
#define Bfly0(a, b)                                 \
{                                                   \
    real tr = dar(b);                               \
    real ti = dai(b);                               \
    dar(b)  = dar(a) - tr; dar(a) += tr;            \
    dai(b)  = dai(a) - ti; dai(a) += ti;            \
}
//  w = (0, 1)
#define Bfly1p(a, b)                                \
{                                                   \
    real tr = dai(b);                               \
    real ti = dar(b);                               \
    dar(b)  = dar(a) + tr; dar(a) -= tr;            \
    dai(b)  = dai(a) - ti; dai(a) += ti;            \
}
//  w = (0, -1)
#define Bfly1m(a, b)                                \
{                                                   \
    real tr = dai(b);                               \
    real ti = dar(b);                               \
    dar(b)  = dar(a) - tr; dar(a) += tr;            \
    dai(b)  = dai(a) + ti; dai(a) -= ti;            \
}
//  w = (S45, S45)
#define Bfly2pp(a, b)                               \
{                                                   \
    real tr = real(S45*(dar(b) - dai(b)));          \
    real ti = real(S45*(dai(b) + dar(b)));          \
    dar(b)  = dar(a) - tr; dar(a) += tr;            \
    dai(b)  = dai(a) - ti; dai(a) += ti;            \
}
//  w = (S45, -S45)
#define Bfly2pm(a, b)                               \
{                                                   \
    real tr = real(S45*(dar(b) + dai(b)));          \
    real ti = real(S45*(dai(b) - dar(b)));          \
    dar(b)  = dar(a) - tr; dar(a) += tr;            \
    dai(b)  = dai(a) - ti; dai(a) += ti;            \
}
//  w = (-S45, S45)
#define Bfly2mp(a, b)                               \
{                                                   \
    real tr = real(S45*(dar(b) + dai(b)));          \
    real ti = real(S45*(dai(b) - dar(b)));          \
    dar(b)  = dar(a) + tr; dar(a) -= tr;            \
    dai(b)  = dai(a) + ti; dai(a) -= ti;            \
}
//  w = (-S45, -S45)
#define Bfly2mm(a, b)                               \
{                                                   \
    real tr = real(S45*(dar(b) - dai(b)));          \
    real ti = real(S45*(dai(b) + dar(b)));          \
    dar(b)  = dar(a) + tr; dar(a) -= tr;            \
    dai(b)  = dai(a) + ti; dai(a) -= ti;            \
}

#define swap(i, j)                                  \
{                                                   \
    real t = dar(i); dar(i) = dar(j); dar(j) = t;   \
         t = dai(i); dai(i) = dai(j); dai(j) = t;   \
}
/*======================================================================*/
static
void    FFTcx1d(real* data, int dim, int sign)
{
    for(int i = 0, j = 0;;)
    {
        if(i < j) 
            swap(i, j);
        if(++i >= dim) 
            break;
        for(int k = dim >> 1; k <= j; j -= k, k >>= 1);
        j += k;
    }
/*......................................................................*/
    {   // three first passes combined (mmax = 1,2,4)
        if(sign)
        {   // normal transform
            for(i = 0; i < dim; i += 8)
            {
                Bfly0  (i,   i+1);  // pass 1
                Bfly0  (i+2, i+3);  // pass 1
                Bfly0  (i,   i+2);  // pass 2
                Bfly1p (i+1, i+3);  // pass 2
                Bfly0  (i+4, i+5);  // pass 1
                Bfly0  (i+6, i+7);  // pass 1
                Bfly0  (i+4, i+6);  // pass 2
                Bfly1p (i+5, i+7);  // pass 2
                Bfly0  (i,   i+4);  // pass 4
                Bfly2pp(i+1, i+5);  // pass 4
                Bfly1p (i+2, i+6);  // pass 4
                Bfly2mp(i+3, i+7);  // pass 4
            }
        }
        else
        {   // inverse transform
            for(i = 0; i < dim; i += 8)
            {
                Bfly0  (i,   i+1);  // pass 1
                Bfly0  (i+2, i+3);  // pass 1
                Bfly0  (i,   i+2);  // pass 2
                Bfly1m (i+1, i+3);  // pass 2
                Bfly0  (i+4, i+5);  // pass 1
                Bfly0  (i+6, i+7);  // pass 1
                Bfly0  (i+4, i+6);  // pass 2
                Bfly1m (i+5, i+7);  // pass 2
                Bfly0  (i,   i+4);  // pass 4
                Bfly2pm(i+1, i+5);  // pass 4
                Bfly1m (i+2, i+6);  // pass 4
                Bfly2mm(i+3, i+7);  // pass 4
            }
        }
    }
/*......................................................................*/
    reall theta = C_PI / 8; 
    if(sign) 
        theta = -theta;
    {
        cdef2(wp, cos(theta), -sin(theta));
        cdef2(w0, 1, 0);
        cdef1(w1, wp);
        cdef(w2); cmul2(w2, w1, wp);
        cdef(w3); cmul2(w3, w2, wp);
        cdef(w4); cmul2(w4, w3, wp);
        cdef(w5); cmul2(w5, w4, wp);
        cdef(w6); cmul2(w6, w5, wp);
        cdef(w7); cmul2(w7, w6, wp);

        if(sign)
            for(i = 0; i < dim; i += 16)
            {
                Bfly0 (i,   i+ 8);
                BflyN (i+1, i+ 9, w1);
                BflyN (i+2, i+10, w2);
                BflyN (i+3, i+11, w3);
                Bfly1p(i+4, i+12);
                BflyN (i+5, i+13, w5);
                BflyN (i+6, i+14, w6);
                BflyN (i+7, i+15, w7);
            }
        else
            for(i = 0; i < dim; i += 16)
            {
                Bfly0 (i,   i+ 8);
                BflyN (i+1, i+ 9, w1);
                BflyN (i+2, i+10, w2);
                BflyN (i+3, i+11, w3);
                Bfly1m(i+4, i+12);
                BflyN (i+5, i+13, w5);
                BflyN (i+6, i+14, w6);
                BflyN (i+7, i+15, w7);
            }
    }

    for(int mmax = 16; mmax < dim; mmax *= 2)
    {
        theta *= 0.5;
        cdef2(wp, cos(theta), -sin(theta));
        cdef2(w0, 1, 0);
        cdef1(w1, wp);

        reall tw = theta * 16.0;
        reall t1 = 0; 

        for(int m = 0;;)
        {
            cdef(w2); cmul2(w2, w1, wp);
            cdef(w3); cmul2(w3, w2, wp);
            cdef(w4); cmul2(w4, w3, wp);
            cdef(w5); cmul2(w5, w4, wp);
            cdef(w6); cmul2(w6, w5, wp);
            cdef(w7); cmul2(w7, w6, wp);
            cdef(w8); cmul2(w8, w7, wp);
            cdef(w9); cmul2(w9, w8, wp);
            cdef(wa); cmul2(wa, w9, wp);
            cdef(wb); cmul2(wb, wa, wp);
            cdef(wc); cmul2(wc, wb, wp);
            cdef(wd); cmul2(wd, wc, wp);
            cdef(we); cmul2(we, wd, wp);
            cdef(wf); cmul2(wf, we, wp);

            for(i = m; i < dim; i += mmax*2)
            {
                BflyN(i+ 0, mmax + i+ 0, w0);
                BflyN(i+ 1, mmax + i+ 1, w1);
                BflyN(i+ 2, mmax + i+ 2, w2);
                BflyN(i+ 3, mmax + i+ 3, w3);
                BflyN(i+ 4, mmax + i+ 4, w4);
                BflyN(i+ 5, mmax + i+ 5, w5);
                BflyN(i+ 6, mmax + i+ 6, w6);
                BflyN(i+ 7, mmax + i+ 7, w7);
                BflyN(i+ 8, mmax + i+ 8, w8);
                BflyN(i+ 9, mmax + i+ 9, w9);
                BflyN(i+10, mmax + i+10, wa);
                BflyN(i+11, mmax + i+11, wb);
                BflyN(i+12, mmax + i+12, wc);
                BflyN(i+13, mmax + i+13, wd);
                BflyN(i+14, mmax + i+14, we);
                BflyN(i+15, mmax + i+15, wf);
            }
            m += 16;
            if(m >= mmax) 
                break;
            t1 += tw;
            cmov2(w0, cos(t1), -sin(t1));
            cmul2(w1, w0, wp);
        }
    }
}
/*======================================================================*/
static
void    FFTcx1dt(real* data, real* sc, int dim, int sign)
{
    for(int i = 0, j = 0;;)
    {
        if(i < j) 
            swap(i, j);
        if(++i >= dim) 
            break;
        for(int k = dim >> 1; k <= j; j -= k, k >>= 1);
        j += k;
    }
/*......................................................................*/
    {   // three first passes combined (mmax = 1,2,4)
        if(sign)
        {   // normal transform
            for(i = 0; i < dim; i += 8)
            {
                Bfly0  (i,   i+1);  // pass 1
                Bfly0  (i+2, i+3);  // pass 1
                Bfly0  (i,   i+2);  // pass 2
                Bfly1p (i+1, i+3);  // pass 2
                Bfly0  (i+4, i+5);  // pass 1
                Bfly0  (i+6, i+7);  // pass 1
                Bfly0  (i+4, i+6);  // pass 2
                Bfly1p (i+5, i+7);  // pass 2
                Bfly0  (i,   i+4);  // pass 4
                Bfly2pp(i+1, i+5);  // pass 4
                Bfly1p (i+2, i+6);  // pass 4
                Bfly2mp(i+3, i+7);  // pass 4
            }
        }
        else
        {   // inverse transform
            for(i = 0; i < dim; i += 8)
            {
                Bfly0  (i,   i+1);  // pass 1
                Bfly0  (i+2, i+3);  // pass 1
                Bfly0  (i,   i+2);  // pass 2
                Bfly1m (i+1, i+3);  // pass 2
                Bfly0  (i+4, i+5);  // pass 1
                Bfly0  (i+6, i+7);  // pass 1
                Bfly0  (i+4, i+6);  // pass 2
                Bfly1m (i+5, i+7);  // pass 2
                Bfly0  (i,   i+4);  // pass 4
                Bfly2pm(i+1, i+5);  // pass 4
                Bfly1m (i+2, i+6);  // pass 4
                Bfly2mm(i+3, i+7);  // pass 4
            }
        }
    }
/*......................................................................*/
    int sci = dim >> 2;
    {
        int sii = sci;
        cdef2(w1, scc(sii), scs(sii)); sii += sci;
        cdef2(w2, scc(sii), scs(sii)); sii += sci;
        cdef2(w3, scc(sii), scs(sii)); sii += sci;
        cdef2(w4, scc(sii), scs(sii)); sii += sci;
        cdef2(w5, scc(sii), scs(sii)); sii += sci;
        cdef2(w6, scc(sii), scs(sii)); sii += sci;
        cdef2(w7, scc(sii), scs(sii));
        
        if(sign)
            for(i = 0; i < dim; i += 16)
            {
                Bfly0 (i,   i+ 8);
                BflyN (i+1, i+ 9, w1);
                BflyN (i+2, i+10, w2);
                BflyN (i+3, i+11, w3);
                Bfly1p(i+4, i+12);
                BflyN (i+5, i+13, w5);
                BflyN (i+6, i+14, w6);
                BflyN (i+7, i+15, w7);
            }
        else
            for(i = 0; i < dim; i += 16)
            {
                Bfly0 (i,   i+ 8);
                BflyN (i+1, i+ 9, w1);
                BflyN (i+2, i+10, w2);
                BflyN (i+3, i+11, w3);
                Bfly1m(i+4, i+12);
                BflyN (i+5, i+13, w5);
                BflyN (i+6, i+14, w6);
                BflyN (i+7, i+15, w7);
            }
    }

    for(int mmax = 16; mmax < dim; mmax *= 2)
    {
        cdef2(w0, 1, 0);
        sci >>= 1;
        for(int m = 0, sii = sci;;)
        {
            cdef2(w1, scc(sii), scs(sii)); sii += sci;
            cdef2(w2, scc(sii), scs(sii)); sii += sci;
            cdef2(w3, scc(sii), scs(sii)); sii += sci;
            cdef2(w4, scc(sii), scs(sii)); sii += sci;
            cdef2(w5, scc(sii), scs(sii)); sii += sci;
            cdef2(w6, scc(sii), scs(sii)); sii += sci;
            cdef2(w7, scc(sii), scs(sii)); sii += sci;
            cdef2(w8, scc(sii), scs(sii)); sii += sci;
            cdef2(w9, scc(sii), scs(sii)); sii += sci;
            cdef2(wa, scc(sii), scs(sii)); sii += sci;
            cdef2(wb, scc(sii), scs(sii)); sii += sci;
            cdef2(wc, scc(sii), scs(sii)); sii += sci;
            cdef2(wd, scc(sii), scs(sii)); sii += sci;
            cdef2(we, scc(sii), scs(sii)); sii += sci;
            cdef2(wf, scc(sii), scs(sii)); sii += sci;

            for(i = m; i < dim; i += mmax*2)
            {
                BflyN(i+ 0, mmax + i+ 0, w0);
                BflyN(i+ 1, mmax + i+ 1, w1);
                BflyN(i+ 2, mmax + i+ 2, w2);
                BflyN(i+ 3, mmax + i+ 3, w3);
                BflyN(i+ 4, mmax + i+ 4, w4);
                BflyN(i+ 5, mmax + i+ 5, w5);
                BflyN(i+ 6, mmax + i+ 6, w6);
                BflyN(i+ 7, mmax + i+ 7, w7);
                BflyN(i+ 8, mmax + i+ 8, w8);
                BflyN(i+ 9, mmax + i+ 9, w9);
                BflyN(i+10, mmax + i+10, wa);
                BflyN(i+11, mmax + i+11, wb);
                BflyN(i+12, mmax + i+12, wc);
                BflyN(i+13, mmax + i+13, wd);
                BflyN(i+14, mmax + i+14, we);
                BflyN(i+15, mmax + i+15, wf);
            }
            m += 16;
            if(m >= mmax) 
                break;
            cmov2(w0, scc(sii), scs(sii)); sii += sci;
        }
    }
}
/*######################################################################*/
static
void    RealFFTcvtt(real* data, real* sc, int dim, int sign)
{
    real s, ms;
    if(sign)
    {
        s  = real( 0.5);
        ms = real(-0.5);
    }
    else
    {
        s  = real(-0.5);
        ms = real( 0.5);
    }
    for(int i = 1, j = dim - 1; i < j; ++i, --j)
    {
        real tr = (dai(i) + dai(j)) * ms;
        real ti = (dar(i) - dar(j)) * s;
        real sr = tcos(i) * tr - tsin(i) * ti;
        real si = tcos(i) * ti + tsin(i) * tr;
        tr = (dar(i) + dar(j)) * real(0.5);
        ti = (dai(i) - dai(j)) * real(0.5);
        dar(i) = tr + sr; dar(j) = tr - sr;
        dai(i) = ti + si; dai(j) = si - ti;
    }
    dai(j) = -dai(j);
}
/*----------------------------------------------------------------------*/
static
void    RealFFTcvt(real* data, int dim, int sign)
{
    real s, ms;
    reall theta = C_PI / double(dim);
    if(sign)
    {
        s  = real( 0.5);
        ms = real(-0.5);
        theta = -theta;
    }
    else
    {
        s  = real(-0.5);
        ms = real( 0.5);
    }
    cdef2(wp, cos(theta), -sin(theta));
    cdef1(wr, wp);
    for(int i = 1, j = dim-1; i < j; ++i, --j)
    {
        real tr = (dai(i) + dai(j)) * ms;
        real ti = (dar(i) - dar(j)) *  s;
        real sr = wrr*tr - wri*ti;
        real si = wrr*ti + wri*tr;
        tr = (dar(i) + dar(j)) * real(0.5);
        ti = (dai(i) - dai(j)) * real(0.5);
        dar(i) = tr + sr; dar(j) = tr - sr;
        dai(i) = ti + si; dai(j) = si - ti;
        cmul2(wr, wr, wp);
    }
    dai(j) = -dai(j);
}
/*######################################################################*/
void  __stdcall InitSinCosTable(real* sc, int dim, int sign)
{
    double rad = 2.0 * C_PI / double(dim);
    if(sign) 
        rad = -rad;
    sc[1] = 0.0;    // -sin
    sc[0] = 1.0;    //  cos

    double w = rad;
    for(int i = 2; i < dim; i += 2, w += rad)
    {
        sc[i+1] = real(-sin(w));
        sc[i+0] = real( cos(w));
    }
}
/*======================================================================*/
void  __stdcall CFFT1(real* data, int dim, int sign, real* sc)
{
    if(sc)
        FFTcx1dt(data, sc, dim, sign);
    else    
        FFTcx1d (data, dim, sign);
}
/*----------------------------------------------------------------------*/
void  __stdcall RFFT1(real* data, int dim, int sign, real* sc)
{
    dim >>= 1;
    if(sc)
    {
        if(sign)
        {
            RealFFTcvtt(data, sc, dim, 1);
            real t = dar(0); 
            dar(0) = (t + dai(0)) * real(0.5); 
            dai(0) = (t - dai(0)) * real(0.5);
            FFTcx1dt(data, sc, dim, 1);
        }
        else
        {
            FFTcx1dt(data, sc, dim, 0);
            RealFFTcvtt(data, sc, dim, 0);
            real t = dar(0); 
            dar(0) = t + dai(0); 
            dai(0) = t - dai(0);
        }
    }
    else
    {
        if(sign)
        {
            RealFFTcvt(data, dim, 1);
            real t = dar(0); 
            dar(0) = (t + dai(0)) * real(0.5); 
            dai(0) = (t - dai(0)) * real(0.5);
            FFTcx1d(data, dim, 1);
        }
        else
        {
            FFTcx1d(data, dim, 0);
            RealFFTcvt(data, dim, 0);
            real t = dar(0); 
            dar(0) = t + dai(0); 
            dai(0) = t - dai(0);
        }
    }
}
/*======================================================================*/
__inline    
double  Pcos(double* a, int na, double dk)
{
    double k = dk;
    double w = a[0];
    for(int i = 1; i < na; ++i, k += dk) 
        w += a[i] * cos(k);
    return w;
}
/*----------------------------------------------------------------------*/
static 
void    PolyCos(real* wnd, int n, double* a, int na, double& mean, double& sum)
{
    double rad = 2.0 * C_PI / double(n);
    double w = Pcos(a, na, rad * double(-n));
    wnd[0] = real(w);
    mean = w * w;
    sum  = w;
    for(int i = 1, n2 = n >> 1; i < n2; ++i)
    {
        w = Pcos(a, na, rad * double(i-n));
        wnd[i] = wnd[n-i] = real(w);
        mean += 2.0 * w * w;
        sum  += w + w;
    }
    w = Pcos(a, na, rad * double(n2-n));
    wnd[n2] = real(w);
    mean += w * w;
    sum  += w;
}
/*----------------------------------------------------------------------*/
void __stdcall  FFTWindow(real* wnd, int n, int window, double& mean, double& sum)
{
    switch(window)
    {
        default :   // Rectangle B = 1.0
        {
            for(int i = 0; i < n; ++i) 
                wnd[i] = 1.0f;
            mean = 0.0;
            sum  = n;
            break;
        }
        case 1  :   // Blackman-Harris 4 term B = 2.01
        {
            double a[4] = { 0.358750, -0.488290, 0.141280, -0.011680 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 2  :   // Nuttall4b 
        {
            double a[4] = { 0.355768, -0.487396, 0.144232, -0.012604 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 3  :   // Nuttall4c 
        {
            double a[4] = { 0.3635819, -0.4891775, 0.1365995, -0.0106411 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 4  :   // HFT95 
        {
            double a[5] = { 1.0, -1.9383379, 1.3045202, -0.4028270, 0.0350665 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 5  :   // HFT90 
        {
            double a[5] = { 1.0, -1.942604, 1.340318, -0.440811, 0.043097 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 6  :   // HFT116 
        {
            double a[6] = { 1.0, -1.9575375, 1.4780705, -0.6367431, 0.1228389, -0.0066288 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 7  :   // HFT144 
        {
            double a[7] = {  1.0, 
                            -1.96760033, 
                             1.57983607, 
                            -0.81123644, 
                             0.22583558, 
                            -0.02773848, 
                             0.00090360 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 8  :   // HFT169 
        {
            double a[8] = {  1.0,
                            -1.97441842,
                             1.65409888,
                            -0.95788186,
                             0.33673420,
                            -0.06364621,
                             0.00521942,
                            -0.00010599 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 9  :   // HFT196 
        {
            double a[9] = {  1.0,
                            -1.979280420,
                             1.710288951,
                            -1.081629853,
                             0.448734314,
                            -0.112376628,
                             0.015122992,
                            -0.000871252,
                             0.000011896 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 10 :   // HFT223
        {
            double a[10] = {  1.0,
                             -1.98298997309,
                              1.75556083063,
                             -1.19037717712,
                              0.56155440797,
                             -0.17296769663,
                              0.03233247087,
                             -0.00324954578,
                              0.00013801040,
                             -0.00000132725 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 11 :   // HFT248D
        {
            double a[11] = {  1.0,
                             -1.985844164102,
                              1.791176438506,
                             -1.282075284005,
                              0.667777530266,
                             -0.240160796576,
                              0.056656381764,
                             -0.008134974479,
                              0.000624544650,
                             -0.000019808998,
                              0.000000132974 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }

        case 12 :   // Blackman-Harris 7 term B = 2.5
        {
            double a[7] = {  2.7105140069342415e-1,
                            -4.3329793923448606e-1,
                             2.1812299954311062e-1,
                            -6.5925446388030898e-2,
                             1.0811742098372268e-2,
                            -7.7658482522509342e-4,
                             1.3887217350903198e-5 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }
        case 13 :   // Cosine summ 7 term
        {
            double a[7] = {  2.712203605850388e-1,
                            -4.334446123274422e-1,
                             2.180041228929303e-1,
                            -6.578534329560609e-2,
                             1.076186730534183e-2,
                            -7.700127105808265e-4,
                             1.368088305992921e-5 };
            PolyCos(wnd, n, a, DIM(a), mean, sum);
            break;
        }         
    }
}
/*######################################################################*/
